writeNamespaceImports <-
function(package, file = "", append = TRUE, quote = FALSE,
         width = 0.9 * getOption("width")) {
    writeImports <- function(x, prefix, file, width) {
        Rkeywords <- 
          c("NULL", "NA", "TRUE", "FALSE", "Inf", "NaN", "NA_integer_",
            "NA_real_", "NA_character_", "NA_complex_", "function", "while",
            "repeat", "for", "if", "in", "else", "next", "break", "...")
        for (i in seq_len(length(x))[!(names(x) %in% ignoredPackages)]) {
            if (quote) {
                qstring1 <- "\""
                qstring2 <- "\""
            } else {
                qstring1 <- ""
                qstring2 <- ifelse(x[[i]] %in% Rkeywords, "\"", "")
                qstring2[grep("(<-|\\[)", x[[i]])] <- "\""
            }
            cat(strwrap(paste(prefix, "(", qstring1, names(x[i]), qstring1, ", ",
                              paste(qstring2, x[[i]], qstring2, sep = "",
                                    collapse = ", "), ")", sep = ""),
                        width = width, exdent = nchar(prefix) + 1),
                file = file, sep = "\n")
            cat("\n", file = file)
        }
    }
    ignoredPackages <- c("base", "Autoloads")
    if (file == "") 
        file <- stdout()
    else if (is.character(file)) {
        file <- file(file, ifelse(append, "a", "w"))
        on.exit(close(file))
    }
    else if (!isOpen(file, "w")) {
        open(file, "w")
        on.exit(close(file))
    }
    if (!inherits(file, "connection")) 
        stop("'file' must be a character string or connection")

    allDeps <- findExternalDeps(package)
    allDepNames <- sort(unique(unlist(lapply(allDeps, names), use.names = FALSE)))
    allDepNames <- allDepNames[!(allDepNames %in% ignoredPackages)]
    if (length(allDepNames) > 0) {
        hasNamespace <-
          unlist(lapply(allDepNames,
                 function(x)
                 !identical(tryCatch(getNamespace(x),
                                     error = function(e) FALSE), FALSE)))
        dependsNames <- allDepNames[!hasNamespace]
        importsNames <- allDepNames[hasNamespace]
        ignoredPackages <- c(ignoredPackages, dependsNames)
        cat("#Generated by codetoolsBioC version ",
            packageDescription("codetoolsBioC")[["Version"]], "\n", file = file,
            sep = "")
        cat("#Timestamp: ", date(), "\n\n", file = file, sep = "")
        if (length(dependsNames) > 0) {
            cat(strwrap(paste("Depends: ",
                              paste(dependsNames, collapse = ", "),
                              sep = ""),
                        width = width, exdent = 9, prefix = "#"),
                file = file, sep = "\n")
            cat("\n", file = file)
        }
        if (length(importsNames) > 0) {
            cat(strwrap(paste("Imports: ",
                              paste(importsNames, collapse = ", "),
                              sep = ""),
                        width = width, exdent = 9, prefix = "#"),
                    file = file, sep = "\n")
            cat("\n", file = file)
        }
        writeImports(allDeps[["S4Classes"]], "importClassesFrom", file, width)
        writeImports(allDeps[["S4Methods"]], "importMethodsFrom", file, width)
        writeImports(allDeps[["functions"]], "importFrom", file, width)
        writeImports(allDeps[["variables"]], "importFrom", file, width)
    }
}
